@@ -1,10 +1,10 @@
 /*
 
-  $Id: cfgreader.c,v 1.50 2004/06/01 21:45:46 pkot Exp $
+  $Id: cfgreader.c,v 1.76 2006/08/09 19:36:58 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,13 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999      Tim Potter
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2000      Jan Derfinak
+  Copyright (C) 2001      Jan Kratochvil
+  Copyright (C) 2001-2005 Pawel Kot
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2005      Bastien Nocera
 
   Config file (/etc/gnokiirc and ~/.gnokiirc) reader.
 
@@ -43,39 +49,28 @@
 #include "cfgreader.h"
 #include "gnokii-internal.h"
 
-API struct gn_cfg_header *gn_cfg_info;
+GNOKII_API struct gn_cfg_header *gn_cfg_info;
 static gn_config gn_config_default, gn_config_global;
 
-/* Read configuration information from a ".INI" style file */
-struct gn_cfg_header *cfg_file_read(const char *filename)
+struct gn_cfg_header *cfg_memory_read(const char **lines)
 {
-	FILE *handle;
-	char *line;
-	char *buf;
+	char *line, *buf;
 	struct gn_cfg_header *cfg_info = NULL, *cfg_head = NULL;
+	int i;
 
-	/* Error check */
-	if (filename == NULL) {
-		return NULL;
-	}
-
-	/* Initialisation */
-	if ((buf = (char *)malloc(255)) == NULL) {
-		return NULL;
-	}
-
-	/* Open file */
-	if ((handle = fopen(filename, "r")) == NULL) {
-		dprintf("cfg_file_read - open %s: %s\n", filename, strerror(errno));
+	/* Check whether the given memory location is not null */
+	if (lines == NULL) {
+		dprintf("cfg_memory_read - passed nil data\n");
 		return NULL;
 	} else {
-		dprintf("Opened configuration file %s\n", filename);
+		dprintf("Opened configuration file from memory\n");
 	}
 
 	/* Iterate over lines in the file */
-	while (fgets(buf, 255, handle) != NULL) {
+	for (i = 0; lines[i] != NULL; i++) {
 
-		line = buf;
+		line = strdup (lines[i]);
+		buf = line;
 
 		/* Strip leading, trailing whitespace */
 		while(isspace((int) *line))
@@ -85,8 +80,10 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 			line[strlen(line) - 1] = '\0';
 
 		/* Ignore blank lines and comments */
-		if ((*line == '\n') || (*line == '\0') || (*line == '#'))
+		if ((*line == '\n') || (*line == '\0') || (*line == '#')) {
+			free(buf);
 			continue;
+		}
 
 		/* Look for "headings" enclosed in square brackets */
 		if ((line[0] == '[') && (line[strlen(line) - 1] == ']')) {
@@ -94,7 +91,6 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 
 			/* Allocate new heading entry */
 			if ((heading = (struct gn_cfg_header *)malloc(sizeof(*heading))) == NULL) {
-				free(buf);
 				return NULL;
 			}
 
@@ -121,19 +117,19 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 
 			dprintf("Added new section %s\n", heading->section);
 
+			free(buf);
+
 			/* Go on to next line */
 			continue;
 		}
 
 		/* Process key/value line */
-
 		if ((strchr(line, '=') != NULL) && cfg_info != NULL) {
 			struct gn_cfg_entry *entry;
 			char *value;
 
 			/* Allocate new entry */
 			if ((entry = (struct gn_cfg_entry *)malloc(sizeof(*entry))) == NULL) {
-				free(buf);
 				return NULL;
 			}
 
@@ -141,10 +137,10 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 			memset(entry, '\0', sizeof(*entry));
 
 			value = strchr(line, '=');
-			*value = '\0';		/* Split string */
+			*value = '\0';	/* Split string */
 			value++;
 
-			while(isspace((int) *value)) {      /* Remove leading white */
+			while (isspace((int) *value)) { /* Remove leading white */
 				value++;
 			}
 
@@ -169,19 +165,147 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 
 			dprintf("Adding key/value %s/%s\n", entry->key, entry->value);
 
+			free(buf);
+
 			/* Go on to next line */
 			continue;
 		}
 
 			/* Line not part of any heading */
-		fprintf(stderr, "Orphaned line: %s\n", line);
+		fprintf(stderr, _("Orphaned line: %s\n"), line);
+
+		free(buf);
 	}
 
-	free(buf);
 	/* Return pointer to configuration information */
 	return cfg_head;
 }
 
+GNOKII_API void gn_cfg_free_default()
+{
+	while (gn_cfg_info) {
+		struct gn_cfg_header *next;
+		struct gn_cfg_entry *entry;
+
+		/* free section name */
+		free(gn_cfg_info->section);
+
+		/* delete all entries */
+		entry = gn_cfg_info->entries;
+		while (entry) {
+			struct gn_cfg_entry *entry_next;
+			free(entry->key);
+			free(entry->value);
+			entry_next = entry->next;
+			free(entry);
+			entry = entry_next;
+		}
+
+		/* proceed with next section */
+		next = gn_cfg_info->next;
+		free(gn_cfg_info);
+		gn_cfg_info = next;
+	}
+}
+
+#define READ_CHUNK_SIZE 64
+
+/* Read configuration information from a ".INI" style file */
+struct gn_cfg_header *cfg_file_read(const char *filename)
+{
+	FILE *handle;
+	char *lines, *line_begin, *line_end, *pos;
+	char **split_lines;
+	int read, ret, num_lines, i, copied;
+	struct gn_cfg_header *header = NULL;
+
+	/* Open file */
+	if ((handle = fopen(filename, "r")) == NULL) {
+		dprintf("cfg_file_read - open %s: %s\n", filename, strerror(errno));
+		goto out;
+	} else {
+		dprintf("Opened configuration file %s\n", filename);
+	}
+
+	/* Read the lines */
+	lines = NULL;
+	read = 0;
+	do {
+		lines = realloc(lines, read + READ_CHUNK_SIZE);
+		if (!lines)
+			goto err_read;
+
+		ret = fread(lines + read, 1, READ_CHUNK_SIZE, handle);
+		/* Read error */
+		if (ret < 0 && feof(handle) == 0)
+			goto err_read;
+
+		/* Overflow */
+		if (read + ret < read)
+			goto err_read;
+
+		read += ret;
+	} while (ret > 0);
+
+	fclose(handle);
+	lines = realloc(lines, read + 1);
+	lines[read] = '\0';
+
+	/* Now split the lines */
+	split_lines = NULL;
+	line_begin = lines;
+	num_lines = 0;
+	copied = 0;
+
+	while ((pos = strchr(line_begin, '\n')) && copied < read) {
+		char *buf;
+
+		if (!pos) {
+			line_end = lines + read;
+		} else {
+			line_end = pos;
+		}
+
+		num_lines++;
+		buf = malloc(line_end - line_begin + 1);
+		strncpy(buf, line_begin, line_end - line_begin);
+		buf[line_end - line_begin] = 0;
+		split_lines = realloc(split_lines,
+				(num_lines + 1) * sizeof(char*));
+		split_lines[num_lines - 1] = buf;
+
+		if (pos) {
+			copied += (line_end + 1 - line_begin);
+			line_begin = line_end + 1;
+		}
+	}
+
+
+	free(lines);
+	if (split_lines == NULL)
+		goto out;
+	split_lines[num_lines] = NULL;
+
+	/* Finally, load the configuration from the split lines */
+	header = cfg_memory_read((const char **)split_lines);
+
+	/* Free the split_lines */
+	for (i = 0; split_lines[i] != NULL; i++) {
+		free(split_lines[i]);
+	}
+	free(split_lines);
+
+	goto out;
+
+err_read:
+	fclose(handle);
+	if (lines)
+		free(lines);
+
+out:
+	return header;
+}
+
 /*  Write configuration information to a config file */
 int cfg_file_write(struct gn_cfg_header *cfg, const char *filename)
 {
@@ -193,7 +317,7 @@ int cfg_file_write(struct gn_cfg_header *cfg, const char *filename)
  * Find the value of a key in a config file.  Return value associated
  * with key or NULL if no such key exists.
  */
-API char *gn_cfg_get(struct gn_cfg_header *cfg, const char *section, const char *key)
+GNOKII_API char *gn_cfg_get(struct gn_cfg_header *cfg, const char *section, const char *key)
 {
 	struct gn_cfg_header *h;
 	struct gn_cfg_entry *e;
@@ -221,10 +345,11 @@ API char *gn_cfg_get(struct gn_cfg_header *cfg, const char *section, const char
 /*
  * Return all the entries of the given section.
  */
-void cfg_foreach(struct gn_cfg_header *cfg, const char *section, cfg_foreach_func func)
+void cfg_foreach(const char *section, cfg_foreach_func func)
 {
 	struct gn_cfg_header *h;
 	struct gn_cfg_entry *e;
+	struct gn_cfg_header *cfg = gn_cfg_info;
 
 	if ((cfg == NULL) || (section == NULL) || (func == NULL)) {
 		return;
@@ -279,40 +404,24 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 
 	memset(cfg, '\0', sizeof(gn_config));
 
+	/* You need to specify at least model and port in the phone section */
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "model")))
-		strcpy(cfg->model, def->model);
+		/* strcpy(cfg->model, def->model); */
+		return false;
 	else
 		snprintf(cfg->model, sizeof(cfg->model), "%s", val);
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "port")))
-		strcpy(cfg->port_device, def->port_device);
+		/* strcpy(cfg->port_device, def->port_device); */
+		return false;
 	else
 		snprintf(cfg->port_device, sizeof(cfg->port_device), "%s", val);
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "connection")))
 		cfg->connection_type = def->connection_type;
 	else {
-		if (!strcasecmp(val, "serial"))
-			cfg->connection_type = GN_CT_Serial;
-		else if (!strcasecmp(val, "dau9p"))
-			cfg->connection_type = GN_CT_DAU9P;
-		else if (!strcasecmp(val, "dlr3p"))
-			cfg->connection_type = GN_CT_DLR3P;
-		else if (!strcasecmp(val, "infrared"))
-			cfg->connection_type = GN_CT_Infrared;
-		else if (!strcasecmp(val, "m2bus"))
-			cfg->connection_type = GN_CT_M2BUS;
-		else if (!strcasecmp(val, "irda"))
-			cfg->connection_type = GN_CT_Irda;
-		else if (!strcasecmp(val, "bluetooth"))
-			cfg->connection_type = GN_CT_Bluetooth;
-#ifndef WIN32
-		else if (!strcasecmp(val, "tcp"))
-			cfg->connection_type = GN_CT_TCP;
-#endif
-		else if (!strcasecmp(val, "tekram"))
-			cfg->connection_type = GN_CT_Tekram;
-		else {
+		cfg->connection_type = gn_get_connectiontype(val);
+		if (cfg->connection_type == GN_CT_NONE) {
 			fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "connection", val);
 			return false;
 		}
@@ -388,6 +497,22 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 		return false;
 	}
 
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "sm_retry")))
+		cfg->sm_retry = def->sm_retry;
+	else if (sscanf(val, " %d %c", &cfg->sm_retry, &ch) != 1) {
+		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "sm_retry", val);
+		return false;
+	}
+
+	/* There is no global setting. You need to set irda_string
+	 * in each section if you want it working */
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "irda_string")))
+		strcpy(cfg->irda_string, "");
+	else {
+		snprintf(cfg->irda_string, sizeof(cfg->irda_string), "%s", val);
+		dprintf("Setting irda_string in section %s to %s\n", section, cfg->irda_string);
+	}
+
 	return true;
 }
 
@@ -411,12 +536,17 @@ static bool cfg_get_log_target(gn_log_target *t, const char *opt)
 }
 
 #define MAX_PATH_LEN 200
-API int gn_cfg_read_default()
+GNOKII_API gn_error gn_cfg_read_default()
 {
 	char *homedir;
 	char rcfile[MAX_PATH_LEN];
 #ifdef WIN32
-	const char *globalrc = "gnokiirc";
+	char globalrc[MAX_PATH_LEN];
+	char *systemroot;
+
+	systemroot = getenv("SYSTEMROOT");
+	strncpy(globalrc, systemroot ? systemroot : "", MAX_PATH_LEN);
+	strncat(globalrc, "\\gnokiirc", MAX_PATH_LEN);
 
 	homedir = getenv("HOMEDRIVE");
 	strncpy(rcfile, homedir ? homedir : "", MAX_PATH_LEN);
@@ -424,7 +554,7 @@ API int gn_cfg_read_default()
 	strncat(rcfile, homedir ? homedir : "", MAX_PATH_LEN);
 	strncat(rcfile, "\\_gnokiirc", MAX_PATH_LEN);
 #else
-	const char *globalrc = "/etc/gnokiirc";
+	const char globalrc[] = "/etc/gnokiirc";
 
 	homedir = getenv("HOME");
 	if (homedir) strncpy(rcfile, homedir, MAX_PATH_LEN);
@@ -437,30 +567,37 @@ API int gn_cfg_read_default()
 		if (gn_cfg_file_read(globalrc)) {
 			/* That failed too so exit */
 			fprintf(stderr, _("Couldn't open %s or %s.\n"), rcfile, globalrc);
-			return -1;
+			return GN_ERR_NOCONFIG;
 		}
 	}
-	return 0;
+	return GN_ERR_NONE;
 }
 
 /* DEPRECATED */
-API int gn_cfg_read(char **bindir)
+GNOKII_API gn_error gn_cfg_read(char **bindir)
 {
-	int retval;
+	gn_error retval;
 
 	retval = gn_cfg_read_default();
 
-	(char *)*bindir = gn_cfg_get(gn_cfg_info, "global", "bindir");
-	if (!*bindir) *bindir = gn_cfg_get(gn_cfg_info, "gnokiid", "bindir");
-	if (!*bindir) *bindir = "/usr/local/sbin";
+	*bindir = gn_cfg_get(gn_cfg_info, "global", "bindir");
+	if (!*bindir)
+		*bindir = gn_cfg_get(gn_cfg_info, "gnokiid", "bindir");
+	if (!*bindir)
+		*bindir = "/usr/local/sbin";
 
 	return retval;
 }
 
-API int gn_cfg_file_read(const char *file)
+static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 {
 	char *val;
 
+	if (file == NULL && lines == NULL) {
+		dprintf("Couldn't open a config file or memory.\n");
+		return GN_ERR_NOCONFIG;
+	}
+
 	/* I know that it doesn't belong here but currently there is now generic
 	 * application init function anywhere.
 	 */
@@ -470,9 +607,18 @@ API int gn_cfg_file_read(const char *file)
 	/*
 	 * Try opening a given config file
 	 */
-	if ((gn_cfg_info = cfg_file_read(file)) == NULL) {
-		fprintf(stderr, _("Couldn't open %s config file,\n"), file);
-		return -1;
+	if (file != NULL)
+		gn_cfg_info = cfg_file_read(file);
+	else
+		gn_cfg_info = cfg_memory_read(lines);
+
+	if (gn_cfg_info == NULL) {
+		/* this is bad, but the previous was much worse - bozo */
+		if (file)
+			dprintf("Couldn't open %s config file.\n", file);
+		else
+			dprintf("Couldn't read config\n");
+		return GN_ERR_NOCONFIG;
 	}
 	strcpy(gn_config_default.model, "");
 	strcpy(gn_config_default.port_device, "");
@@ -483,13 +629,15 @@ API int gn_cfg_file_read(const char *file)
 	gn_config_default.hardware_handshake = false;
 	gn_config_default.require_dcd = false;
 	gn_config_default.smsc_timeout = -1;
+	strcpy(gn_config_default.irda_string, "");
 	strcpy(gn_config_default.connect_script, "");
 	strcpy(gn_config_default.disconnect_script, "");
 	gn_config_default.rfcomm_cn = 1;
+	gn_config_default.sm_retry = 0;
 
 	if (!cfg_psection_load(&gn_config_global, "global", &gn_config_default)) {
-		fprintf(stderr, _("No global section in % config file.\n"), file);
-		return -2;
+		fprintf(stderr, _("No [global] section in %s config file.\n"), file);
+		return GN_ERR_NOPHONE;
 	}
 
 	/* hack to support [sms] / smsc_timeout parameter */
@@ -500,36 +648,47 @@ API int gn_cfg_file_read(const char *file)
 			gn_config_global.smsc_timeout = 10 * atoi(val);
 	}
 
-	if (!cfg_get_log_target(&gn_log_debug_mask, "debug")) return -2;
-	if (!cfg_get_log_target(&gn_log_rlpdebug_mask, "rlpdebug")) return -2;
-	if (!cfg_get_log_target(&gn_log_xdebug_mask, "xdebug")) return -2;
+	if (!cfg_get_log_target(&gn_log_debug_mask, "debug") ||
+	    !cfg_get_log_target(&gn_log_rlpdebug_mask, "rlpdebug") ||
+	    !cfg_get_log_target(&gn_log_xdebug_mask, "xdebug"))
+		return GN_ERR_NOLOG;
 
 	gn_log_debug("LOG: debug mask is 0x%x\n", gn_log_debug_mask);
 	gn_log_rlpdebug("LOG: rlpdebug mask is 0x%x\n", gn_log_rlpdebug_mask);
 	gn_log_xdebug("LOG: xdebug mask is 0x%x\n", gn_log_xdebug_mask);
-	return 0;
+	return GN_ERR_NONE;
 }
 
-API bool gn_cfg_phone_load(const char *iname, struct gn_statemachine *state)
+GNOKII_API gn_error gn_cfg_file_read(const char *file)
+{
+	return cfg_file_or_memory_read(file, NULL);
+}
+
+GNOKII_API gn_error gn_cfg_memory_read(const char **lines)
+{
+	return cfg_file_or_memory_read(NULL, lines);
+}
+
+GNOKII_API gn_error gn_cfg_phone_load(const char *iname, struct gn_statemachine *state)
 {
 	char section[256];
 
-	if (iname == NULL || *iname == '\0')
+	if (iname == NULL || *iname == '\0') {
 		state->config = gn_config_global;
-	else {
+	} else {
 		snprintf(section, sizeof(section), "phone_%s", iname);
 		if (!cfg_psection_load(&state->config, section, &gn_config_global))
-			return false;
+			return GN_ERR_NOPHONE;
 	}
 
 	if (state->config.model[0] == '\0') {
 		fprintf(stderr, _("Config error - no model specified.\n"));
-		return false;
+		return GN_ERR_NOMODEL;
 	}
 	if (state->config.port_device[0] == '\0') {
 		fprintf(stderr, _("Config error - no port specified.\n"));
-		return false;
+		return GN_ERR_NOPORT;
 	}
 
-	return true;
+	return GN_ERR_NONE;
 }
