@@ -1,10 +1,10 @@
 /*
 
-  $Id: gsm-encoding.c,v 1.58 2004/06/30 22:16:49 pkot Exp $
+  $Id: gsm-encoding.c,v 1.72 2006/07/31 20:38:13 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,11 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2001 Pawe Kot <pkot@linuxnews.pl>
+  Copyright (C) 1999-2000 Pavel Janik ml.
+  Copyright (C) 2001-2005 Pawel Kot
+  Copyright (C) 2002      Markus Plail, Manfred Jonsson
+  Copyright (C) 2002-2004 BORBELY Zoltan
+  Copyright (C) 2003      Martin Goldhahn
 
   Functions for encoding SMS, calendar and other things.
 
@@ -44,6 +48,13 @@
 #ifdef HAVE_LANGINFO_CODESET
 #  include <langinfo.h>
 #endif
+#ifdef HAVE_LOCALE_CHARSET
+#  include <libcharset.h>
+#else
+/* FIXME: We should include here somehow ../intl/localcharset.h, but it may
+ * cause problems with MSVC. */
+extern const char *locale_charset(void); /* from ../intl/localcharset.c */
+#endif
 
 static const char *base64_alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
@@ -78,7 +89,7 @@ static unsigned char gsm_default_alphabet[GN_CHAR_ALPHABET_SIZE] = {
 
 static unsigned char gsm_reverse_default_alphabet[256];
 static bool reversed = false;
-extern const char *locale_charset(void); /* from ../intl/localcharset.c */
+static char application_encoding[64] = "";
 
 static void tbl_setup_reverse()
 {
@@ -97,6 +108,103 @@ static bool char_is_escape(unsigned char value)
 	return (value == GN_CHAR_ESCAPE);
 }
 
+static const char *get_langinfo_codeset(void)
+{
+	static const char *codeset = NULL;
+
+	if (!codeset) {
+#ifdef HAVE_LANGINFO_CODESET
+		codeset = nl_langinfo(CODESET);
+#else
+#  ifdef WIN32
+		/* As suggested by Ben Bryant, http://codesnipers.com/?q=node/46 */
+		char szCP[10];
+		sprintf(szCP, ".%d", GetACP());
+		codeset = setlocale(LC_ALL, szCP);
+#  else
+		codeset = locale_charset();
+#  endif
+#endif
+	}
+	return codeset;
+}
+
+GNOKII_API const char *gn_char_get_encoding()
+{
+	if (*application_encoding)
+		return application_encoding; /* app has overriden encoding setting */
+	else
+		return get_langinfo_codeset(); /* return default codeset */
+}
+
+/* detect the correct len of a string (also for multibyte chars like "umlaute") */
+int char_mblen(const char *src)
+{
+	int len = mbstowcs(NULL, src, 0);
+	dprintf("char_mblen(%s): %i\n", src, len);
+	return len;
+}
+
+static int char_mbtowc(wchar_t *dst, const char *src, int maxlen, MBSTATE *mbs)
+{
+#ifdef HAVE_ICONV
+	size_t nconv;
+	char *pin;
+	char *pout;
+	size_t inlen;
+	size_t outlen;
+	iconv_t cd;
+
+	pin = (char *)src;
+	pout = (char *)dst;
+	inlen = 4;
+	outlen = 4;
+
+	cd = iconv_open("WCHAR_T", gn_char_get_encoding());
+	nconv = iconv(cd, &pin, &inlen, &pout, &outlen);
+	iconv_close(cd);
+
+	return (char*)dst == pout ? -1 : pin-src;
+#else
+#  ifdef HAVE_WCRTOMB
+        return mbrtowc(dst, src, maxlen, mbs);
+#  else
+        return mbtowc(dst, src, maxlen);
+#  endif
+#endif
+}
+
+static int char_wctomb(char *dst, wchar_t src, MBSTATE *mbs)
+{
+#ifdef HAVE_ICONV
+	size_t nconv;
+	char *pin;
+	char *pout;
+	size_t inlen;
+	size_t outlen;
+	iconv_t cd;
+
+	pin = (char *)&src;
+	pout = (char *)dst;
+	inlen = 4;
+	outlen = 4;
+
+	cd = iconv_open(gn_char_get_encoding(), "WCHAR_T");
+	nconv = iconv(cd, &pin, &inlen, &pout, &outlen);
+	if (nconv == (size_t)-1)
+		perror("iconv");
+	iconv_close(cd);
+
+	return nconv == -1 ? -1 : pout-dst;
+#else
+    #ifdef HAVE_WCRTOMB
+	return wcrtomb(dst, src, mbs);
+    #else
+	return wctomb(dst, src);
+    #endif
+#endif
+}
+
 /*
  * In GSM specification there are 10 characters in the extension
  * of the default alphabet. Their values look a bit random, they are
@@ -108,7 +216,7 @@ static bool char_def_alphabet_ext(unsigned char value)
 {
 	wchar_t retval;
 
-	if (mbtowc(&retval, &value, 1) == -1) return false;
+	if (char_mbtowc(&retval, &value, 1, NULL) == -1) return false;
 	return (value == 0x0c ||
 		value == '^' ||
 		value == '{' ||
@@ -155,7 +263,7 @@ static unsigned char char_def_alphabet_ext_encode(unsigned char value)
 	}
 }
 
-API bool gn_char_def_alphabet(unsigned char *string)
+GNOKII_API bool gn_char_def_alphabet(unsigned char *string)
 {
 	unsigned int i, len = strlen(string);
 
@@ -276,6 +384,7 @@ skip:
 	return (out_num - output);
 }
 
+/* null terminates the string */
 void char_ascii_decode(unsigned char* dest, const unsigned char* src, int len)
 {
 	int i, j;
@@ -305,6 +414,7 @@ unsigned int char_ascii_encode(unsigned char* dest, const unsigned char* src, un
 	return i;
 }
 
+/* null terminates the string */
 void char_hex_decode(unsigned char* dest, const unsigned char* src, int len)
 {
 	int i;
@@ -333,11 +443,7 @@ int char_uni_alphabet_encode(unsigned char const *value, wchar_t *dest, MBSTATE
 {
 	int length;
 
-#ifdef HAVE_WCRTOMB
-	switch (length = mbrtowc(dest, value, MB_CUR_MAX, mbs)) {
-#else
-	switch (length = mbtowc(dest, value, MB_CUR_MAX)) {
-#endif
+	switch (length = char_mbtowc(dest, value, MB_CUR_MAX, mbs)) {
 	case -1:
 		dprintf("Error calling mctowb!\n");
 		*dest = '?';
@@ -351,11 +457,7 @@ int char_uni_alphabet_decode(wchar_t value, unsigned char *dest, MBSTATE *mbs)
 {
 	int length;
 
-#ifdef HAVE_WCRTOMB
-	switch (length = wcrtomb(dest, value, mbs)) {
-#else
-	switch (length = wctomb(dest, value)) {
-#endif
+    switch (length = char_wctomb(dest, value, mbs)) {
 	case -1:
 		*dest = '?';
 		length = 1;
@@ -364,6 +466,7 @@ int char_uni_alphabet_decode(wchar_t value, unsigned char *dest, MBSTATE *mbs)
 	}
 }
 
+/* null terminates the string */
 void char_ucs2_decode(unsigned char* dest, const unsigned char* src, int len)
 {
 	int i_len = 0, o_len = 0, length;
@@ -405,15 +508,19 @@ void char_ucs2_encode(unsigned char* dest, const unsigned char* src, int len)
 		case -1:
 			i_len++;
 			break;
+		case 0:
+			/* return at the end of the string */
+			return;
 		default:
 			i_len += length;
 			break;
 		}
-		sprintf(dest + (o_len << 2), "%04lx", wc);
+		sprintf(dest + (o_len << 2), "%-4lc", wc);
 	}
 	return;
 }
 
+/* null terminates the string */
 unsigned int char_unicode_decode(unsigned char* dest, const unsigned char* src, int len)
 {
 	int i, length = 0, pos = 0;
@@ -421,7 +528,8 @@ unsigned int char_unicode_decode(unsigned char* dest, const unsigned char* src,
 
 	MBSTATE_DEC_CLEAR(mbs);
 	for (i = 0; i < len / 2; i++) {
-		length = char_uni_alphabet_decode((src[i * 2] << 8) | src[(i * 2) + 1], dest, &mbs);
+		wchar_t wc = src[i * 2] << 8 | src[(i * 2) + 1];
+		length = char_uni_alphabet_decode(wc, dest, &mbs);
 		dest += length;
 		pos += length;
 	}
@@ -502,7 +610,7 @@ void bin2hex(unsigned char *dest, const unsigned char *src, unsigned int len)
    destination number) for SMS sending function. */
 int char_semi_octet_pack(char *number, unsigned char *output, gn_gsm_number_type type)
 {
-	unsigned char *in_num = number;  /* Pointer to the input number */
+	char *in_num = number;  /* Pointer to the input number */
 	unsigned char *out_num = output; /* Pointer to the output */
 	int count = 0; /* This variable is used to notify us about count of already
 			  packed numbers. */
@@ -513,8 +621,9 @@ int char_semi_octet_pack(char *number, unsigned char *output, gn_gsm_number_type
 	   only international and unknown number. */
 
 	*out_num++ = type;
-	if (type == GN_GSM_NUMBER_International) in_num++; /* Skip '+' */
-	if ((type == GN_GSM_NUMBER_Unknown) && (*in_num == '+')) in_num++; /* Optional '+' in Unknown number type */
+
+	if ((type == GN_GSM_NUMBER_International || type == GN_GSM_NUMBER_Unknown) && *in_num == '+')
+		in_num++; /* skip leading '+' */
 
 	/* The next field is the number. It is in semi-octet representation - see
 	   GSM scpecification 03.40 version 6.1.0, section 9.1.2.3, page 31. */
@@ -543,7 +652,7 @@ char *char_bcd_number_get(u8 *number)
 {
 	static char buffer[GN_BCD_STRING_MAX_LENGTH] = "";
 	int length = number[0]; /* This is the length of BCD coded number */
-	int count, digit;
+	int count, digit, i = 0;
 
 	if (length > GN_BCD_STRING_MAX_LENGTH) length = GN_BCD_STRING_MAX_LENGTH;
 	memset(buffer, 0, GN_BCD_STRING_MAX_LENGTH);
@@ -554,6 +663,7 @@ char *char_bcd_number_get(u8 *number)
 		break;
 	case GN_GSM_NUMBER_International:
 		sprintf(buffer, "+");
+		i++;
 		if (length == GN_BCD_STRING_MAX_LENGTH) length--; /* avoid overflow */
 	case GN_GSM_NUMBER_Unknown:
 	case GN_GSM_NUMBER_National:
@@ -563,27 +673,21 @@ char *char_bcd_number_get(u8 *number)
 	default:
 		for (count = 0; count < length - 1; count++) {
 			digit = number[count+2] & 0x0f;
-			if (digit < 10) sprintf(buffer, "%s%d", buffer, digit);
+			if (digit < 10)
+				buffer[i++] = digit + '0';
 			digit = number[count+2] >> 4;
-			if (digit < 10) sprintf(buffer, "%s%d", buffer, digit);
+			if (digit < 10)
+				buffer[i++] = digit + '0';
 		}
 		break;
 	}
 	return buffer;
 }
 
-static const char *get_langinfo_codeset(void)
+void gn_char_set_encoding(const char* encoding)
 {
-	static const char *codeset = NULL;
-
-	if (!codeset) {
-#ifdef HAVE_LANGINFO_CODESET
-		codeset = nl_langinfo(CODESET);
-#else
-		codeset = locale_charset();
-#endif
-	}
-	return codeset;
+	strncpy(application_encoding, encoding, sizeof(application_encoding) - 1);
+	application_encoding[sizeof(application_encoding) - 1] = 0;
 }
 
 /* UTF-8 conversion functions */
@@ -599,7 +703,7 @@ int utf8_decode(char *outstring, size_t outlen, const char *instring, size_t inl
 	pin = (char *)instring;
 	pout = outstring;
 
-	cd = iconv_open(get_langinfo_codeset(), "UTF-8");
+	cd = iconv_open(gn_char_get_encoding(), "UTF-8");
 	nconv = iconv(cd, &pin, &inlen, &pout, &outlen);
 	iconv_close(cd);
 	*pout = 0;
@@ -655,7 +759,7 @@ int utf8_encode(char *outstring, int outlen, const char *instring, int inlen)
 	pin = (char *)instring;
 	pout = outstring;
 
-	cd = iconv_open("UTF-8", get_langinfo_codeset());
+	cd = iconv_open("UTF-8", gn_char_get_encoding());
 
 	nconv = iconv(cd, &pin, &inleft, &pout, &outleft);
 	*pout = 0;
@@ -851,3 +955,84 @@ int utf8_base64_decode(char *dest, int destlen, const char *in, int inlen)
 	free(aux);
 	return retval;
 }
+
+/* We are escaping the following characters (according to rfc 2426):
+ * '\n', '\r', ';', ',', '\'.
+ */
+int add_slashes(char *dest, char *src, int maxlen, int len)
+{
+	int i, j;
+
+	for (i = 0, j = 0; i < len && j < maxlen; i++, j++) {
+		switch (src[i]) {
+		case '\n':
+			dest[j++] = '\\';
+			dest[j] = 'n';
+			break;
+		case '\r':
+			dest[j++] = '\\';
+			dest[j] = 'r';
+			break;
+		case '\\':
+		case ';':
+		case ',':
+			dest[j++] = '\\';
+		default:
+			dest[j] = src[i];
+			break;
+		}
+	}
+	dest[j] = 0;
+	return j;
+}
+
+int strip_slashes(char *dest, char *src, int maxlen, int len)
+{
+	int i, j, slash_state = 0;
+
+	for (i = 0, j = 0; i < len && j < maxlen; i++) {
+		switch (src[i]) {
+		case ';':
+		case ',':
+			if (slash_state) {
+				dest[j++] = src[i];
+				slash_state = 0;
+			}
+			break;
+		case '\\':
+			if (slash_state) {
+				dest[j++] = src[i];
+				slash_state = 0;
+			} else {
+				slash_state = 1;
+			}
+			break;
+		case 'n':
+			if (slash_state) {
+				dest[j++] = '\n';
+				slash_state = 0;
+			} else {
+				dest[j++] = src[i];
+			}
+			break;
+		case 'r':
+			if (slash_state) {
+				dest[j++] = '\r';
+				slash_state = 0;
+			} else {
+				dest[j++] = src[i];
+			}
+			break;
+		default:
+			if (slash_state) {
+				dest[j++] = '\\';
+				slash_state = 0;
+			} else {
+				dest[j++] = src[i];
+			}
+			break;
+		}
+	}
+	dest[j] = 0;
+	return j;
+}
