@@ -1,10 +1,10 @@
 /*
 
-  $Id: gsm-sms.c,v 1.143 2004/05/08 20:52:39 bozo Exp $
+  $Id: gsm-sms.c,v 1.151 2006/08/08 20:06:29 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,9 +22,12 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2001-2002 Pawe Kot <pkot@linuxnews.pl>
-  Copyright (C) 2001-2002 Pavel Machek <pavel@ucw.cz>
-
+  Copyright (C) 2001-2003 Pawel Kot
+  Copyright (C) 2001-2002 Markus Plail, Pavel Machek <pavel@ucw.cz>
+  Copyright (C) 2002-2003 Ladis Michl
+  Copyright (C) 2003-2004 BORBELY Zoltan
+  Copyright (C) 2006      Igor Popik
+  
   Library for parsing and creating Short Messages (SMS).
 
 */
@@ -78,6 +81,7 @@ static struct sms_udh_data headers[] = {
 	{ 0x04, "\x01\x02\x01\x00" },         /* Fax Messages */
 	{ 0x04, "\x01\x02\x02\x00" },         /* Email Messages */
 	{ 0x06, "\x05\x04\x0b\x84\x23\xf0" }, /* WAP PUSH */
+	{ 0x06, "\x05\x04\x0b\x84\x0b\x84" },
 	{ 0x00, "" },
 };
 
@@ -108,14 +112,14 @@ static void sms_default(gn_sms *sms)
 	sms->dcs.u.general.m_class = 0;
 }
 
-API void gn_sms_default_submit(gn_sms *sms)
+GNOKII_API void gn_sms_default_submit(gn_sms *sms)
 {
 	sms_default(sms);
 	sms->type = GN_SMS_MT_Submit;
 	sms->memory_type = GN_MT_SM;
 }
 
-API void gn_sms_default_deliver(gn_sms *sms)
+GNOKII_API void gn_sms_default_deliver(gn_sms *sms)
 {
 	sms_default(sms);
 	sms->type = GN_SMS_MT_Deliver;
@@ -138,7 +142,9 @@ static char *sms_timestamp_print(u8 *number)
 {
 #ifdef DEBUG
 #define LOCAL_DATETIME_MAX_LENGTH 23
-	static unsigned char buffer[LOCAL_DATETIME_MAX_LENGTH] = "";
+	static char buffer[LOCAL_DATETIME_MAX_LENGTH];
+	char buf[5];
+	int i;
 
 	if (!number) return NULL;
 
@@ -146,25 +152,33 @@ static char *sms_timestamp_print(u8 *number)
 
 	/* Ugly hack, but according to the GSM specs, the year is stored
          * as the 2 digit number. */
-	if (number[0] < 70) sprintf(buffer, "20");
-	else sprintf(buffer, "19");
+	if ((10*(number[0] & 0x0f) + (number[0] >> 4)) < 70)
+		sprintf(buffer, "20");
+	else
+		sprintf(buffer, "19");
 
-	sprintf(buffer, "%s%d%d-", buffer, number[0] & 0x0f, number[0] >> 4);
-	sprintf(buffer, "%s%d%d-", buffer, number[1] & 0x0f, number[1] >> 4);
-	sprintf(buffer, "%s%d%d ", buffer, number[2] & 0x0f, number[2] >> 4);
-	sprintf(buffer, "%s%d%d:", buffer, number[3] & 0x0f, number[3] >> 4);
-	sprintf(buffer, "%s%d%d:", buffer, number[4] & 0x0f, number[4] >> 4);
-	sprintf(buffer, "%s%d%d",  buffer, number[5] & 0x0f, number[5] >> 4);
+	for (i = 0; i < 6; i++) {
+		int c;
+		char buf2[4];
+		switch (i) {
+		case 0: case 1: c = '-'; break;
+		case 3: case 4: c = ':'; break;
+		default: c = ' '; break;
+		}
+		snprintf(buf2, 4, "%d%d%c", number[i] & 0x0f, number[i] >> 4, c);
+		strcat(buffer, buf2);
+	}
 
 	/* The GSM spec is not clear what is the sign of the timezone when the
 	 * 6th bit is set. Some SMSCs are compatible with our interpretation,
 	 * some are not. If your operator does use incompatible SMSC and wrong
 	 * sign disturbs you, change the sign here.
 	 */
-	if (number[6] & 0x08) sprintf(buffer, "%s-", buffer);
-	else sprintf(buffer, "%s+", buffer);
+	if (number[6] & 0x08) strcat(buffer, "-");
+	else strcat(buffer, "+");
 	/* The timezone is given in quarters. The base is GMT. */
-	sprintf(buffer, "%s%02d00", buffer, (10 * (number[6] & 0x07) + (number[6] >> 4)) / 4);
+	sprintf(buf, "%02d00", (10 * (number[6] & 0x07) + (number[6] >> 4)) / 4);
+	strcat(buffer, buf);
 
 	return buffer;
 #undef LOCAL_DATETIME_MAX_LENGTH
@@ -507,6 +521,10 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 				dprintf("WAP vCalendarSecure\n");
 				udh->udh[nr].type = GN_SMS_UDH_WAPvCalendarSecure;
 				break;
+			case 0x0b84:
+				dprintf("WAP Push\n");
+				udh->udh[nr].type = GN_SMS_UDH_WAPPush;
+				break;
 			default:
 				dprintf("Unknown\n");
 				udh->udh[nr].type = GN_SMS_UDH_Unknown;
@@ -687,7 +705,7 @@ static gn_error sms_pdu_decode(gn_sms_raw *rawsms, gn_sms *sms)
  * the highlevel SMS. In data->raw_sms there's SMS read by the phone
  * driver, data->sms is the place for the parsed SMS.
  */
-API gn_error gn_sms_parse(gn_data *data)
+GNOKII_API gn_error gn_sms_parse(gn_data *data)
 {
 	if (!data->raw_sms || !data->sms) return GN_ERR_INTERNALERROR;
 	/* Let's assume at the moment that all messages are PDU coded */
@@ -703,7 +721,7 @@ API gn_error gn_sms_parse(gn_data *data)
  * phone driver. Not that raw_sms field in the gn_data structure must
  * be initialized
  */
-API gn_error gn_sms_request(gn_data *data, struct gn_statemachine *state)
+GNOKII_API gn_error gn_sms_request(gn_data *data, struct gn_statemachine *state)
 {
 	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
 	return gn_sm_functions(GN_OP_GetSMS, data, state);
@@ -717,7 +735,7 @@ API gn_error gn_sms_request(gn_data *data, struct gn_statemachine *state)
  * This function is the frotnend for reading SMS. Note that SMS field
  * in the gn_data structure must be initialized.
  */
-API gn_error gn_sms_get(gn_data *data, struct gn_statemachine *state)
+GNOKII_API gn_error gn_sms_get(gn_data *data, struct gn_statemachine *state)
 {
 	gn_error error;
 	gn_sms_raw rawsms;
@@ -741,7 +759,7 @@ API gn_error gn_sms_get(gn_data *data, struct gn_statemachine *state)
  * This function is the frontend for deleting SMS. Note that SMS field
  * in the gn_data structure must be initialized.
  */
-API gn_error gn_sms_delete(gn_data *data, struct gn_statemachine *state)
+GNOKII_API gn_error gn_sms_delete(gn_data *data, struct gn_statemachine *state)
 {
 	gn_sms_raw rawsms;
 
@@ -762,7 +780,7 @@ static gn_error sms_request_no_validate(gn_data *data, struct gn_statemachine *s
 	return gn_sm_functions(GN_OP_GetSMSnoValidate, data, state);
 }
 
-API gn_error gn_sms_get_no_validate(gn_data *data, struct gn_statemachine *state)
+GNOKII_API gn_error gn_sms_get_no_validate(gn_data *data, struct gn_statemachine *state)
 {
 	gn_error error;
 	gn_sms_raw rawsms;
@@ -778,7 +796,7 @@ API gn_error gn_sms_get_no_validate(gn_data *data, struct gn_statemachine *state
 	return gn_sms_parse(data);
 }
 
-API gn_error gn_sms_delete_no_validate(gn_data *data, struct gn_statemachine *state)
+GNOKII_API gn_error gn_sms_delete_no_validate(gn_data *data, struct gn_statemachine *state)
 {
 	gn_sms_raw rawsms;
 
@@ -886,7 +904,7 @@ static gn_error sms_verify_status(gn_data *data)
 }
 
 
-API gn_error gn_sms_get_folder_changes(gn_data *data, struct gn_statemachine *state, int has_folders)
+GNOKII_API gn_error gn_sms_get_folder_changes(gn_data *data, struct gn_statemachine *state, int has_folders)
 {
 	gn_error error;
 	gn_sms_folder  sms_folder;
@@ -1266,7 +1284,7 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state);
  * and then phone driver takes the fields it needs and sends it in the
  * phone specific way to the phone.
  */
-API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
+GNOKII_API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
 {
 	gn_error error = GN_ERR_NONE;
 
@@ -1300,7 +1318,7 @@ API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
 	return error;
 }
 
-API int gn_sms_udh_add(gn_sms *sms, gn_sms_udh_type type)
+GNOKII_API int gn_sms_udh_add(gn_sms *sms, gn_sms_udh_type type)
 {
 	sms->udh.length += headers[type].length;
 	sms->udh.udh[sms->udh.number].type = type;
@@ -1400,7 +1418,7 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 	return GN_ERR_NONE;
 }
 
-API gn_error gn_sms_save(gn_data *data, struct gn_statemachine *state)
+GNOKII_API gn_error gn_sms_save(gn_data *data, struct gn_statemachine *state)
 {
 	gn_error error = GN_ERR_NONE;
 	gn_sms_raw rawsms;
@@ -1440,3 +1458,145 @@ cleanup:
 	data->raw_sms = NULL;
 	return error;
 }
+
+/* WAPPush functions */
+
+char *encode_attr_inline_string(char token, char *string, int *data_len) 
+{
+	char *data = NULL;
+	
+	/* we need 3 extra bytes for tags */
+	*data_len = strlen(string) + 3;
+	data = malloc(*data_len);
+
+	if (!data) {
+	    return NULL;
+	}
+	
+	data[0] = token;
+	data[1] = TAG_INLINE;
+	memcpy(data + 2, string, strlen(string));
+	data[*data_len - 1] = 0x00;
+
+	return data;
+}
+
+char *encode_indication(gn_wap_push *wp, int *data_len) 
+{
+	char *data = NULL;
+	char *attr = NULL;
+	int attr_len = 0;
+	int offset = 0;
+	
+	/* encode tag attribute */
+	attr = encode_attr_inline_string(ATTR_HREF, wp->url, &attr_len);
+	
+	if (!attr || !attr_len) {
+	    return NULL;
+	}
+	
+	/* need 5 extra bytes for indication token & attributes */
+	*data_len = attr_len + strlen(wp->text) + 5;
+	data = malloc(*data_len);
+	
+	if (!data) {
+	    return NULL;
+	}
+	
+	/* indication tag token */
+	data[offset++] = TOKEN_KNOWN_AC | TAG_INDICATION;
+	
+	/* attribute */
+	memcpy(data + offset, attr, attr_len);
+	offset += attr_len;
+	data[offset++] = TAG_END;
+
+	/* wappush text */
+	data[offset++] = TAG_INLINE;
+	memcpy(data + offset, wp->text, strlen(wp->text));
+	offset += strlen(wp->text);
+	data[offset++] = 0x00;
+	
+	/* tag end */
+	data[offset++] = TAG_END;
+
+	free(attr);
+
+	return data;
+}
+
+char *encode_si(gn_wap_push *wp, int *data_len)
+{
+	char *data = NULL;
+	char *child = NULL;
+	int child_len = 0;
+	
+	child = encode_indication(wp, &child_len);
+
+	if (!child || !data_len) {
+	    return NULL;
+	}
+
+	/* we need extra 2 bytes for si token */
+	*data_len = child_len + 2;
+	data = malloc(*data_len);
+	
+	if (!data) {
+	    free(child);
+	    return NULL;
+	}
+	
+	data[0] = TOKEN_KNOWN_C | TAG_SI;
+	memcpy(data + 1, child, child_len);
+	data[*data_len - 1] = TAG_END;
+	
+	free(child);
+	
+	return data;
+}
+
+GNOKII_API gn_error gn_wap_push_encode(gn_wap_push *wp) 
+{
+	
+	char *data = NULL;
+	int data_len = 0;
+	
+	data = encode_si(wp, &data_len);
+	
+	if (!data || !data_len) {
+	    return GN_ERR_FAILED;
+	}
+	
+	wp->data = malloc(data_len + sizeof(gn_wap_push_header));
+
+	if (!wp->data) {
+	    return GN_ERR_FAILED;
+	}
+	
+	memcpy(wp->data, &wp->header, sizeof(gn_wap_push_header));
+	memcpy(wp->data + sizeof(gn_wap_push_header), data, data_len);
+	
+	wp->data_len = data_len + sizeof(gn_wap_push_header);
+
+	return GN_ERR_NONE;
+}
+
+GNOKII_API void gn_wap_push_init(gn_wap_push *wp) 
+{
+
+	if (!wp) {
+	    return;
+	}
+
+	memset(wp, 0, sizeof(gn_wap_push));
+	
+	wp->header.wsp_tid 		= 0x00;
+	wp->header.wsp_pdu 		= PDU_TYPE_Push;
+	wp->header.wsp_hlen 		= 0x01;
+	wp->header.wsp_content_type 	= CONTENT_TYPE;
+
+	wp->header.version 	= WBXML_VERSION;
+	wp->header.public_id 	= TAG_SI;
+	wp->header.charset 	= WAPPush_CHARSET;
+	wp->header.stl 		= 0x00; /* string table length */
+}
