@@ -1,10 +1,10 @@
 /*
 
-  $Id: gnapplet.c,v 1.15 2004/06/27 20:27:03 pkot Exp $
+  $Id: gnapplet.c,v 1.32 2006/04/29 15:06:36 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2004 BORBELY Zoltan
+  Copyright (C) 2004-2005 BORBELY Zoltan
 
   This file provides functions specific to the gnapplet series.
   See README for more details on supported mobile phones.
@@ -48,7 +48,7 @@
 #include "gnokii-internal.h"
 #include "gnokii.h"
 
-#define	DRVINSTANCE(s) ((gnapplet_driver_instance *)((s)->driver.driver_instance))
+#define	DRVINSTANCE(s) (*((gnapplet_driver_instance **)(&(s)->driver.driver_instance)))
 #define	FREE(p) do { free(p); (p) = NULL; } while (0)
 
 #define	REQUEST_DEFN(n) \
@@ -95,10 +95,29 @@ static gn_error gnapplet_sms_folder_list(gn_data *data, struct gn_statemachine *
 static gn_error gnapplet_sms_folder_status(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_folder_create(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_folder_delete(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_sms_message_read_nv(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_message_read(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_message_write(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_message_send(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_sms_message_delete_nv(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_sms_message_delete(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_sms_message_move_nv(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_sms_message_move(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_center_read(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_sms_center_write(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_calendar_note_read(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_calendar_note_write(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_calendar_note_delete(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_calendar_todo_read(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_calendar_todo_write(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_calendar_todo_delete(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_calendar_todo_delete_all(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_clock_datetime_read(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_clock_datetime_write(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_clock_alarm_read(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_clock_alarm_write(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_profile_read(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_profile_active_read(gn_data *data, struct gn_statemachine *state);
 
 static gn_error gnapplet_incoming_info(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_incoming_phonebook(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
@@ -106,6 +125,9 @@ static gn_error gnapplet_incoming_netinfo(int messagetype, unsigned char *messag
 static gn_error gnapplet_incoming_power(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_incoming_debug(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_incoming_sms(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_incoming_calendar(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_incoming_clock(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_incoming_profile(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
 
 static gn_incoming_function_type gnapplet_incoming_functions[] = {
 	{ GNAPPLET_MSG_INFO,		gnapplet_incoming_info },
@@ -114,6 +136,9 @@ static gn_incoming_function_type gnapplet_incoming_functions[] = {
 	{ GNAPPLET_MSG_POWER,		gnapplet_incoming_power },
 	{ GNAPPLET_MSG_DEBUG,		gnapplet_incoming_debug },
 	{ GNAPPLET_MSG_SMS,		gnapplet_incoming_sms },
+	{ GNAPPLET_MSG_CALENDAR,	gnapplet_incoming_calendar },
+	{ GNAPPLET_MSG_CLOCK,		gnapplet_incoming_clock },
+	{ GNAPPLET_MSG_PROFILE,		gnapplet_incoming_profile },
 	{ 0,				NULL}
 };
 
@@ -122,7 +147,7 @@ gn_driver driver_gnapplet = {
 	pgen_incoming_default,
 	/* Mobile phone information */
 	{
-		"gnapplet|series60|3650|6600|sx1",	/* Supported models */
+		"gnapplet|symbian|3650|6600|sx1",	/* Supported models */
 		0,			/* Max RF Level */
 		100,			/* Min RF Level */
 		GN_RF_Percentage,	/* RF level units */
@@ -160,7 +185,7 @@ static unsigned char gnapplet_get_semi(const unsigned char *addr)
 	if (!addr[0]) return 0;
 
 	l = 2 * (addr[0] - 1);
-	return (addr[addr[0]] & 0xf0 == 0xf0) ? l - 1 : l;
+	return ((addr[addr[0]] & 0xf0) == 0xf0) ? l - 1 : l;
 }
 
 
@@ -431,14 +456,52 @@ static gn_error gnapplet_functions(gn_operation op, gn_data *data, struct gn_sta
 		return gnapplet_sms_folder_create(data, state);
 	case GN_OP_DeleteSMSFolder:
 		return gnapplet_sms_folder_delete(data, state);
+	case GN_OP_GetSMSnoValidate:
+		return gnapplet_sms_message_read_nv(data, state);
 	case GN_OP_GetSMS:
 		return gnapplet_sms_message_read(data, state);
 	case GN_OP_SaveSMS:
 		return gnapplet_sms_message_write(data, state);
 	case GN_OP_SendSMS:
 		return gnapplet_sms_message_send(data, state);
+	case GN_OP_DeleteSMSnoValidate:
+		return gnapplet_sms_message_delete_nv(data, state);
+	case GN_OP_DeleteSMS:
+		return gnapplet_sms_message_delete(data, state);
+	/*
+	case GN_OP_MoveSMSnoValidate:
+		return gnapplet_sms_message_move_nv(data, state);
+	case GN_OP_MoveSMS:
+		return gnapplet_sms_message_move(data, state);
+	*/
 	case GN_OP_GetSMSCenter:
 		return gnapplet_sms_center_read(data, state);
+	case GN_OP_SetSMSCenter:
+		return gnapplet_sms_center_write(data, state);
+	case GN_OP_GetCalendarNote:
+		return gnapplet_calendar_note_read(data, state);
+	case GN_OP_WriteCalendarNote:
+		return gnapplet_calendar_note_write(data, state);
+	case GN_OP_DeleteCalendarNote:
+		return gnapplet_calendar_note_delete(data, state);
+	case GN_OP_GetToDo:
+		return gnapplet_calendar_todo_read(data, state);
+	case GN_OP_WriteToDo:
+		return gnapplet_calendar_todo_write(data, state);
+	case GN_OP_DeleteAllToDos:
+		return gnapplet_calendar_todo_delete_all(data, state);
+	case GN_OP_GetDateTime:
+		return gnapplet_clock_datetime_read(data, state);
+	case GN_OP_SetDateTime:
+		return gnapplet_clock_datetime_write(data, state);
+	case GN_OP_GetAlarm:
+		return gnapplet_clock_alarm_read(data, state);
+	case GN_OP_SetAlarm:
+		return gnapplet_clock_alarm_write(data, state);
+	case GN_OP_GetProfile:
+		return gnapplet_profile_read(data, state);
+	case GN_OP_GetActiveProfile:
+		return gnapplet_profile_active_read(data, state);
 	default:
 		dprintf("gnapplet unimplemented operation: %d\n", op);
 		return GN_ERR_NOTIMPLEMENTED;
@@ -507,7 +570,7 @@ static gn_error gnapplet_identify(gn_data *data, struct gn_statemachine *state)
 			return error;
 	}
 
-	if (data->manufacturer) snprintf(data->manufacturer, 20, "%s", drvinst->manufacturer);
+	if (data->manufacturer) snprintf(data->manufacturer, GN_MANUFACTURER_MAX_LENGTH, "%s", drvinst->manufacturer);
 	if (data->model) snprintf(data->model, GN_MODEL_MAX_LENGTH, "%s", drvinst->model);
 	if (data->imei) snprintf(data->imei, GN_IMEI_MAX_LENGTH, "%s", drvinst->imei);
 	if (data->revision) snprintf(data->revision, GN_REVISION_MAX_LENGTH, "SW %s, HW %s", drvinst->sw_version, drvinst->hw_version);
@@ -553,7 +616,6 @@ static gn_error gnapplet_incoming_info(int messagetype, unsigned char *message,
 
 static gn_error gnapplet_read_phonebook(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->phonebook_entry) return GN_ERR_INTERNALERROR;
@@ -568,7 +630,6 @@ static gn_error gnapplet_read_phonebook(gn_data *data, struct gn_statemachine *s
 
 static gn_error gnapplet_write_phonebook(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	gn_phonebook_subentry *se;
 	int i, need_defnumber;
 	REQUEST_DEF;
@@ -615,7 +676,6 @@ static gn_error gnapplet_write_phonebook(gn_data *data, struct gn_statemachine *
 
 static gn_error gnapplet_delete_phonebook(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->phonebook_entry) return GN_ERR_INTERNALERROR;
@@ -630,7 +690,6 @@ static gn_error gnapplet_delete_phonebook(gn_data *data, struct gn_statemachine
 
 static gn_error gnapplet_memory_status(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->memory_status) return GN_ERR_INTERNALERROR;
@@ -653,13 +712,14 @@ static gn_error gnapplet_incoming_phonebook(int messagetype, unsigned char *mess
 
 	case GNAPPLET_MSG_PHONEBOOK_READ_RESP:
 		if (!(entry = data->phonebook_entry)) return GN_ERR_INTERNALERROR;
-		entry->empty = false;
+		entry->empty = true;
 		entry->caller_group = 5;
 		entry->name[0] = '\0';
 		entry->number[0] = '\0';
 		entry->subentries_count = 0;
 		memset(&entry->date, 0, sizeof(entry->date));
 		if (error != GN_ERR_NONE) return error;
+		entry->empty = false;
 		n = pkt_get_uint16(&pkt);
 		assert(n < GN_PHONEBOOK_SUBENTRIES_MAX_NUMBER);
 		for (i = 0; i < n; i++) {
@@ -731,7 +791,6 @@ static gn_error gnapplet_incoming_phonebook(int messagetype, unsigned char *mess
 
 static gn_error gnapplet_get_network_info(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->network_info) return GN_ERR_INTERNALERROR;
@@ -744,7 +803,6 @@ static gn_error gnapplet_get_network_info(gn_data *data, struct gn_statemachine
 
 static gn_error gnapplet_get_rf_level(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->rf_unit || !data->rf_level) return GN_ERR_INTERNALERROR;
@@ -794,7 +852,6 @@ static gn_error gnapplet_incoming_netinfo(int messagetype, unsigned char *messag
 
 static gn_error gnapplet_get_power_info(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->battery_unit && !data->battery_level && !data->power_source) return GN_ERR_INTERNALERROR;
@@ -852,7 +909,6 @@ static gn_error gnapplet_incoming_debug(int messagetype, unsigned char *message,
 
 static gn_error gnapplet_sms_get_status(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->sms_status) return GN_ERR_INTERNALERROR;
@@ -865,7 +921,6 @@ static gn_error gnapplet_sms_get_status(gn_data *data, struct gn_statemachine *s
 
 static gn_error gnapplet_sms_folder_list(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->sms_folder_list) return GN_ERR_INTERNALERROR;
@@ -878,7 +933,6 @@ static gn_error gnapplet_sms_folder_list(gn_data *data, struct gn_statemachine *
 
 static gn_error gnapplet_sms_folder_status(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->sms_folder) return GN_ERR_INTERNALERROR;
@@ -892,7 +946,6 @@ static gn_error gnapplet_sms_folder_status(gn_data *data, struct gn_statemachine
 
 static gn_error gnapplet_sms_folder_create(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->sms_folder) return GN_ERR_INTERNALERROR;
@@ -906,7 +959,6 @@ static gn_error gnapplet_sms_folder_create(gn_data *data, struct gn_statemachine
 
 static gn_error gnapplet_sms_folder_delete(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->sms_folder) return GN_ERR_INTERNALERROR;
@@ -938,17 +990,12 @@ static gn_error gnapplet_sms_validate(gn_data *data, struct gn_statemachine *sta
 }
 
 
-static gn_error gnapplet_sms_message_read(gn_data *data, struct gn_statemachine *state)
+static gn_error gnapplet_sms_message_read_nv(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
-	gn_error error;
 	REQUEST_DEF;
 
 	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
 
-	if ((error = gnapplet_sms_validate(data, state)) != GN_ERR_NONE) return error;
-	data->raw_sms->number = data->sms_folder->locations[data->raw_sms->number - 1];
-
 	pkt_put_uint16(&pkt, GNAPPLET_MSG_SMS_MESSAGE_READ_REQ);
 	pkt_put_uint16(&pkt, data->raw_sms->memory_type);
 	pkt_put_uint32(&pkt, data->raw_sms->number);
@@ -957,9 +1004,21 @@ static gn_error gnapplet_sms_message_read(gn_data *data, struct gn_statemachine
 }
 
 
+static gn_error gnapplet_sms_message_read(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error error;
+
+	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
+
+	if ((error = gnapplet_sms_validate(data, state)) != GN_ERR_NONE) return error;
+	data->raw_sms->number = data->sms_folder->locations[data->raw_sms->number - 1];
+
+	return gnapplet_sms_message_read_nv(data, state);
+}
+
+
 static gn_error gnapplet_sms_message_write(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	unsigned char buf[256];
 	gn_error error;
 	int n;
@@ -984,7 +1043,6 @@ static gn_error gnapplet_sms_message_write(gn_data *data, struct gn_statemachine
 
 static gn_error gnapplet_sms_message_send(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	unsigned char buf[256];
 	gn_error error;
 	int n;
@@ -1003,9 +1061,63 @@ static gn_error gnapplet_sms_message_send(gn_data *data, struct gn_statemachine
 }
 
 
+static gn_error gnapplet_sms_message_delete_nv(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_SMS_MESSAGE_DELETE_REQ);
+	pkt_put_uint16(&pkt, data->raw_sms->memory_type);
+	pkt_put_uint32(&pkt, data->raw_sms->number);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_SMS);
+}
+
+
+static gn_error gnapplet_sms_message_delete(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error error;
+
+	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
+
+	if ((error = gnapplet_sms_validate(data, state)) != GN_ERR_NONE) return error;
+	data->raw_sms->number = data->sms_folder->locations[data->raw_sms->number - 1];
+
+	return gnapplet_sms_message_delete_nv(data, state);
+}
+
+
+static gn_error gnapplet_sms_message_move_nv(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_SMS_MESSAGE_MOVE_REQ);
+	pkt_put_uint16(&pkt, data->raw_sms->memory_type);
+	pkt_put_uint32(&pkt, data->raw_sms->number);
+	pkt_put_uint16(&pkt, 0); /* FIXME */
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_SMS);
+}
+
+
+static gn_error gnapplet_sms_message_move(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error error;
+
+	if (!data->raw_sms) return GN_ERR_INTERNALERROR;
+
+	if ((error = gnapplet_sms_validate(data, state)) != GN_ERR_NONE) return error;
+	data->raw_sms->number = data->sms_folder->locations[data->raw_sms->number - 1];
+
+	return gnapplet_sms_message_move_nv(data, state);
+}
+
+
 static gn_error gnapplet_sms_center_read(gn_data *data, struct gn_statemachine *state)
 {
-	gnapplet_driver_instance *drvinst = DRVINSTANCE(state);
 	REQUEST_DEF;
 
 	if (!data->message_center) return GN_ERR_INTERNALERROR;
@@ -1017,6 +1129,27 @@ static gn_error gnapplet_sms_center_read(gn_data *data, struct gn_statemachine *
 }
 
 
+static gn_error gnapplet_sms_center_write(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->message_center) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_SMS_CENTER_WRITE_REQ);
+	pkt_put_uint16(&pkt, data->message_center->id - 1);
+	pkt_put_string(&pkt, data->message_center->name);
+	pkt_put_int16(&pkt, data->message_center->default_name);
+	pkt_put_uint8(&pkt, data->message_center->format);
+	pkt_put_uint8(&pkt, data->message_center->validity);
+	pkt_put_uint8(&pkt, data->message_center->smsc.type);
+	pkt_put_string(&pkt, data->message_center->smsc.number);
+	pkt_put_uint8(&pkt, data->message_center->recipient.type);
+	pkt_put_string(&pkt, data->message_center->recipient.number);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_SMS);
+}
+
+
 static gn_error gnapplet_incoming_sms(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state)
 {
 	gn_sms_folder_list *folders;
@@ -1093,11 +1226,21 @@ static gn_error gnapplet_incoming_sms(int messagetype, unsigned char *message, i
 		if (error != GN_ERR_NONE) return error;
 		break;
 		
+	case GNAPPLET_MSG_SMS_MESSAGE_DELETE_RESP:
+		if (!(rawsms = data->raw_sms)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		break;
+		
+	case GNAPPLET_MSG_SMS_MESSAGE_MOVE_RESP:
+		if (!(rawsms = data->raw_sms)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		break;
+		
 	case GNAPPLET_MSG_SMS_CENTER_READ_RESP:
 		if (!(smsc=data->message_center)) return GN_ERR_INTERNALERROR;
 		if (error != GN_ERR_NONE) return error;
 		memset(smsc, 0, sizeof(*smsc));
-		smsc->id = pkt_get_uint16(&pkt);
+		smsc->id = pkt_get_uint16(&pkt) + 1;
 		pkt_get_string(smsc->name, sizeof(smsc->name), &pkt);
 		smsc->default_name = pkt_get_int16(&pkt);
 		smsc->format = pkt_get_uint8(&pkt);
@@ -1107,6 +1250,360 @@ static gn_error gnapplet_incoming_sms(int messagetype, unsigned char *message, i
 		smsc->recipient.type = pkt_get_uint8(&pkt);
 		pkt_get_string(smsc->recipient.number, sizeof(smsc->recipient.number), &pkt);
 		break;
+		
+	case GNAPPLET_MSG_SMS_CENTER_WRITE_RESP:
+		if (!(smsc=data->message_center)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		smsc->id = pkt_get_uint16(&pkt) + 1;
+		break;
+
+	default:
+		return GN_ERR_UNHANDLEDFRAME;
+	}
+
+	return GN_ERR_NONE;
+}
+
+
+static gn_error gnapplet_calendar_note_read(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->calnote) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CALENDAR_NOTE_READ_REQ);
+	pkt_put_uint32(&pkt, data->calnote->location);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CALENDAR);
+}
+
+
+static gn_error gnapplet_calendar_note_write(gn_data *data, struct gn_statemachine *state)
+{
+	gn_timestamp null_ts;
+	REQUEST_DEF;
+
+	if (!data->calnote) return GN_ERR_INTERNALERROR;
+
+	memset(&null_ts, 0, sizeof(null_ts));
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CALENDAR_NOTE_WRITE_REQ);
+	pkt_put_uint32(&pkt, data->calnote->location);
+	pkt_put_uint8(&pkt, data->calnote->type);
+	pkt_put_timestamp(&pkt, &data->calnote->time);
+	if (data->calnote->end_time.year)
+		pkt_put_timestamp(&pkt, &data->calnote->end_time);
+	else
+		pkt_put_timestamp(&pkt, &null_ts);
+	if (data->calnote->alarm.enabled && data->calnote->alarm.timestamp.year != 0)
+		pkt_put_timestamp(&pkt, &data->calnote->alarm.timestamp);
+	else {
+		pkt_put_timestamp(&pkt, &null_ts);
+	}
+	pkt_put_string(&pkt, data->calnote->text);
+	if (data->calnote->type == GN_CALNOTE_CALL)
+		pkt_put_string(&pkt, data->calnote->phone_number);
+	else
+		pkt_put_string(&pkt, "");
+	if (data->calnote->type == GN_CALNOTE_MEETING)
+		pkt_put_string(&pkt, data->calnote->mlocation);
+	else
+		pkt_put_string(&pkt, "");
+	pkt_put_uint16(&pkt, data->calnote->recurrence);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CALENDAR);
+}
+
+
+static gn_error gnapplet_calendar_note_delete(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->calnote) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CALENDAR_NOTE_DELETE_REQ);
+	pkt_put_uint32(&pkt, data->calnote->location);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CALENDAR);
+}
+
+
+static gn_error gnapplet_calendar_todo_read(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->todo) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CALENDAR_TODO_READ_REQ);
+	pkt_put_uint32(&pkt, data->todo->location);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CALENDAR);
+}
+
+
+static gn_error gnapplet_calendar_todo_write(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->todo) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CALENDAR_TODO_WRITE_REQ);
+	pkt_put_uint32(&pkt, data->todo->location);
+	pkt_put_string(&pkt, data->todo->text);
+	pkt_put_uint8(&pkt, data->todo->priority);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CALENDAR);
+}
+
+
+static gn_error gnapplet_calendar_todo_delete(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->todo) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CALENDAR_TODO_DELETE_REQ);
+	pkt_put_uint32(&pkt, data->todo->location);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CALENDAR);
+}
+
+
+static gn_error gnapplet_calendar_todo_delete_all(gn_data *data, struct gn_statemachine *state)
+{
+	gn_error error;
+	gn_todo todo;
+	gn_todo *todo_save;
+
+	todo_save = data->todo;
+	memset(&todo, 0, sizeof(todo));
+	data->todo = &todo;
+
+	do {
+		data->todo->location = 1;
+		error = gnapplet_calendar_todo_delete(data, state);
+	} while (error == GN_ERR_NONE);
+
+	data->todo = todo_save;
+
+	return GN_ERR_NONE;
+}
+
+
+static gn_error gnapplet_incoming_calendar(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state)
+{
+	gn_calnote *cal;
+	gn_todo *todo;
+	REPLY_DEF;
+
+	switch (code) {
+
+	case GNAPPLET_MSG_CALENDAR_NOTE_READ_RESP:
+		if (!(cal = data->calnote)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		cal->location = pkt_get_uint32(&pkt);
+		cal->type = pkt_get_uint8(&pkt);
+		pkt_get_timestamp(&cal->time, &pkt);
+		pkt_get_timestamp(&cal->end_time, &pkt);
+		pkt_get_timestamp(&cal->alarm.timestamp, &pkt);
+		cal->alarm.enabled = (cal->alarm.timestamp.year != 0);
+		pkt_get_string(cal->text, sizeof(cal->text), &pkt);
+		pkt_get_string(cal->phone_number, sizeof(cal->phone_number), &pkt);
+		pkt_get_string(cal->mlocation, sizeof(cal->mlocation), &pkt);
+		cal->recurrence = pkt_get_uint16(&pkt);
+		break;
+
+	case GNAPPLET_MSG_CALENDAR_NOTE_WRITE_RESP:
+		if (!(cal = data->calnote)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		cal->location = pkt_get_uint32(&pkt);
+		break;
+
+	case GNAPPLET_MSG_CALENDAR_NOTE_DELETE_RESP:
+		if (!(cal = data->calnote)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		cal->location = pkt_get_uint32(&pkt);
+		break;
+
+	case GNAPPLET_MSG_CALENDAR_TODO_READ_RESP:
+		if (!(todo = data->todo)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		todo->location = pkt_get_uint32(&pkt);
+		pkt_get_string(todo->text, sizeof(todo->text), &pkt);
+		todo->priority = pkt_get_uint8(&pkt);
+		break;
+
+	case GNAPPLET_MSG_CALENDAR_TODO_WRITE_RESP:
+		if (!(todo = data->todo)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		todo->location = pkt_get_uint32(&pkt);
+		break;
+
+	case GNAPPLET_MSG_CALENDAR_TODO_DELETE_RESP:
+		if (!(todo = data->todo)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		todo->location = pkt_get_uint32(&pkt);
+		break;
+
+	default:
+		return GN_ERR_UNHANDLEDFRAME;
+	}
+
+	return GN_ERR_NONE;
+}
+
+
+static gn_error gnapplet_clock_datetime_read(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->datetime) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CLOCK_DATETIME_READ_REQ);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CLOCK);
+}
+
+
+static gn_error gnapplet_clock_datetime_write(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->datetime) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CLOCK_DATETIME_WRITE_REQ);
+	pkt_put_timestamp(&pkt, data->datetime);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CLOCK);
+}
+
+
+static gn_error gnapplet_clock_alarm_read(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->alarm) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CLOCK_ALARM_READ_REQ);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CLOCK);
+}
+
+
+static gn_error gnapplet_clock_alarm_write(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->alarm) return GN_ERR_INTERNALERROR;
+	if (!data->alarm->enabled) {
+		data->alarm->timestamp.hour = 0;
+		data->alarm->timestamp.minute = 0;
+		data->alarm->timestamp.second = 0;
+	}
+	data->alarm->timestamp.year = 0;
+	data->alarm->timestamp.month = 1;
+	data->alarm->timestamp.day = 1;
+	data->alarm->timestamp.timezone = 0;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_CLOCK_ALARM_WRITE_REQ);
+	pkt_put_bool(&pkt, data->alarm->enabled);
+	pkt_put_timestamp(&pkt, &data->alarm->timestamp);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_CLOCK);
+}
+
+
+static gn_error gnapplet_incoming_clock(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state)
+{
+	REPLY_DEF;
+
+	switch (code) {
+
+	case GNAPPLET_MSG_CLOCK_DATETIME_READ_RESP:
+		if (!data->datetime) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		pkt_get_timestamp(data->datetime, &pkt);
+		break;
+
+	case GNAPPLET_MSG_CLOCK_DATETIME_WRITE_RESP:
+		if (data->datetime) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		break;
+
+	case GNAPPLET_MSG_CLOCK_ALARM_READ_RESP:
+		if (!data->alarm) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		data->alarm->enabled = pkt_get_bool(&pkt);
+		pkt_get_timestamp(&data->alarm->timestamp, &pkt);
+		break;
+
+	case GNAPPLET_MSG_CLOCK_ALARM_WRITE_RESP:
+		if (data->alarm) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		break;
+
+	default:
+		return GN_ERR_UNHANDLEDFRAME;
+	}
+
+	return GN_ERR_NONE;
+}
+
+
+static gn_error gnapplet_profile_read(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->profile) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_PROFILE_READ_REQ);
+	pkt_put_uint16(&pkt, data->profile->number);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_PROFILE);
+}
+
+
+static gn_error gnapplet_profile_active_read(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->profile) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_PROFILE_GET_ACTIVE_REQ);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_PROFILE);
+}
+
+
+static gn_error gnapplet_incoming_profile(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state)
+{
+	gn_profile *profile;
+	REPLY_DEF;
+
+	switch (code) {
+
+	case GNAPPLET_MSG_PROFILE_READ_RESP:
+		if (!(profile = data->profile)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		profile->number = pkt_get_uint16(&pkt);
+		pkt_get_string(profile->name, sizeof(profile->name), &pkt);
+		profile->default_name = pkt_get_int16(&pkt);
+		profile->keypad_tone = pkt_get_uint8(&pkt);
+		profile->lights = 0;
+		profile->call_alert = pkt_get_uint8(&pkt);
+		profile->ringtone = 0; //!!!FIXME
+		profile->volume = pkt_get_uint8(&pkt);
+		profile->message_tone = 0;
+		profile->warning_tone = pkt_get_uint8(&pkt);
+		profile->vibration = pkt_get_uint8(&pkt);
+		profile->caller_groups = 0;
+		profile->automatic_answer = 0;
+		break;
+
+	case GNAPPLET_MSG_PROFILE_GET_ACTIVE_RESP:
+		if (!(profile = data->profile)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		profile->number = pkt_get_uint16(&pkt);
+		break;
 
 	default:
 		return GN_ERR_UNHANDLEDFRAME;
