@@ -1,10 +1,10 @@
 /*
 
-  $Id: gnokiid.c,v 1.40 2004/06/01 21:45:50 pkot Exp $
+  $Id: gnokiid.c,v 1.48 2006/08/14 14:47:33 dforsi Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,10 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000  Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 2001-2004  Pawel Kot
+  Copyright (C) 2002       Manfred Jonsson
+  Copyright (C) 2002-2003  BORBELY Zoltan
 
   Mainline code for gnokiid daemon. Handles command line parsing and
   various daemon functions.
@@ -40,11 +43,11 @@
 #include <string.h>
 
 #include "misc.h"
-#include "gnokii.h"
+#include "gnokii-internal.h"
 
 /* Global variables */
 bool DebugMode;		/* When true, run in debug mode */
-char *BinDir;		/* Directory of the mgnokiidev command */
+const char *BinDir;	/* Directory of the mgnokiidev command */
 char *lockfile = NULL;
 bool GTerminateThread;
 
@@ -67,7 +70,7 @@ static void version()
 static void usage()
 {
 
-	fprintf(stderr, _("   usage: gnokiid {--help|--version}\n"
+	fprintf(stderr, _("   usage: gnokiid [--help|--version|--debug]\n"
 "          --help            display usage information\n"
 "          --version         displays version and copyright information\n"
 "          --debug           uses stdin/stdout for virtual modem comms\n"));
@@ -78,6 +81,7 @@ static void busterminate(void)
 {
 	gn_vm_terminate();
 	if (lockfile) gn_device_unlock(lockfile);
+	gn_lib_library_free();
 }
 
 /* Main function - handles command line arguments, passes them to separate
@@ -85,7 +89,7 @@ static void busterminate(void)
 
 int main(int argc, char *argv[])
 {
-	char *aux;
+	const char *aux;
 	static bool atexit_registered = false;
 	struct gn_statemachine temp_state;
 
@@ -96,11 +100,8 @@ int main(int argc, char *argv[])
 
 	short_version();
 
-	if (gn_cfg_read_default() < 0) {
-		exit(-1);
-	}
-	BinDir = gn_cfg_get(gn_cfg_info, "global", "bindir");
-	if (!BinDir) BinDir = gn_cfg_get(gn_cfg_info, "gnokiid", "bindir");
+	BinDir = gn_lib_cfg_get("global", "bindir");
+	if (!BinDir) BinDir = gn_lib_cfg_get("gnokiid", "bindir");
 	if (!BinDir) BinDir = "/usr/local/sbin";
 
 	if (!gn_cfg_phone_load("", &temp_state)) exit(-1);
@@ -123,24 +124,24 @@ int main(int argc, char *argv[])
 		DebugMode = false;
 	}
 
-	/* register cleanup function */
-	if (!atexit_registered) {
-		atexit_registered = true;
-		atexit(busterminate);
-	}
-
 	gn_elog_handler = NULL;
 
-	aux = gn_cfg_get(gn_cfg_info, "global", "use_locking");
+	aux = gn_lib_cfg_get("global", "use_locking");
 	/* Defaults to 'no' */
 	if (aux && !strcmp(aux, "yes")) {
 		lockfile = gn_device_lock(temp_state.config.port_device);
 		if (lockfile == NULL) {
-			fprintf(stderr, _("Lock file error. Exiting\n"));
+			fprintf(stderr, _("Lock file error. Exiting.\n"));
 			exit(1);
 		}
 	}
 
+	/* register cleanup function */
+	if (!atexit_registered) {
+		atexit_registered = true;
+		atexit(busterminate);
+	}
+
 	while (1) {
 		if (gn_vm_initialise("", BinDir, DebugMode, true) == false) {
 			exit (-1);
