@@ -1,10 +1,10 @@
 /*
 
-  $Id: gnokii-internal.h,v 1.24 2004/06/30 22:16:48 pkot Exp $
+  $Id: gnokii-internal.h,v 1.34 2006/08/06 20:42:21 pkot Exp $
 
   G N O K I I
 
-  A Linux/Unix toolset and driver for Nokia mobile phones.
+  A Linux/Unix toolset and driver for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,7 +22,11 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2002 Pawe Kot
+  Copyright (C) 1999-2000 Hugh Blemings, Pavel Janik
+  Copyright (C) 2000-2004 Pawel Kot, BORBELY Zoltan
+  Copyright (C) 2000      Marcin Wiacek, Chris Kemp
+  Copyright (C) 2002      Ladis Michl
+  Copyright (C) 2003      Osma Suominen, Martin Goldhahn
 
   Internal API for libgnokii
 
@@ -34,8 +38,16 @@
 #include "config.h"
 #include "compat.h"
 #include "misc.h"
+
+#if !defined(GNOKII_DEPRECATED)
+#  define GNOKII_DEPRECATED /* do not warn about deprecated functions here */
+#endif
+
 #include "gnokii.h"
 
+/* utils */
+gn_connection_type gn_get_connectiontype(const char *connection_type_string);
+
 /* SMS */
 gn_error sms_parse(int offset, gn_data *data);
 gn_error sms_request(gn_data *data, struct gn_statemachine *state);
@@ -56,7 +68,7 @@ gn_error sm_block(int waitfor, gn_data *data, struct gn_statemachine *state);
 gn_error sm_block_no_retry_timeout(int waitfor, int t, gn_data *data, struct gn_statemachine *state);
 gn_error sm_block_no_retry(int waitfor, gn_data *data, struct gn_statemachine *state);
 gn_error sm_block_ack(struct gn_statemachine *state);
-void sm_message_dump(int messagetype, unsigned char *message, int length);
+void sm_message_dump(gn_log_func_t lfunc, int messagetype, unsigned char *message, int length);
 void sm_unhandled_frame_dump(int messagetype, unsigned char *message, int length, struct gn_statemachine *state);
 
 extern void hex2bin(unsigned char *dest, const unsigned char *src, unsigned int len);
@@ -67,6 +79,8 @@ int char_7bit_unpack(unsigned int offset, unsigned int in_length, unsigned int o
 int char_7bit_pack(unsigned int offset, unsigned char *input, unsigned char *output,
 		   unsigned int *in_len);
 
+int char_mblen(const char *src);
+
 unsigned int char_unicode_decode(unsigned char* dest, const unsigned char* src, int len);
 unsigned int char_unicode_encode(unsigned char* dest, const unsigned char* src, int len);
 
@@ -134,6 +148,7 @@ int imelody_sms_encode(unsigned char *imelody, unsigned char *message);
 gn_error phonebook_decode(unsigned char *blockstart, int length,
 			  gn_data *data, int blocks, int memtype, int speeddial_pos);
 gn_error calnote_decode(unsigned char *message, int length, gn_data *data);
+gn_error calnote_get_alarm(int alarmdiff, gn_timestamp *time, gn_timestamp *alarm);
 
 int sms_nokia_smart_message_part_pack(unsigned char *msg, unsigned int size,
 				      unsigned int type, bool first);
@@ -142,7 +157,7 @@ int sms_nokia_bitmap_encode(gn_bmp *bitmap, unsigned char *message, bool first);
 
 struct gn_cfg_header *cfg_file_read(const char *filename);
 typedef void (*cfg_foreach_func)(const char *section, const char *key, const char *value);
-void cfg_foreach(struct gn_cfg_header *cfg, const char *section, cfg_foreach_func func);
+void cfg_foreach(const char *section, cfg_foreach_func func);
 char *cfg_set(struct gn_cfg_header *cfg, const char *section, const char *key, const char *value);
 int cfg_file_write(struct gn_cfg_header *cfg, const char *filename);
 
@@ -158,4 +173,7 @@ int base64_encode(char *dest, int destlen, const char *src, int inlen);
 int utf8_base64_decode(char *dest, int destlen, const char *src, int inlen);
 int utf8_base64_encode(char *dest, int destlen, const char *src, int inlen);
 
+int add_slashes(char *dest, char *src, int maxlen, int len);
+int strip_slashes(char *dest, char *src, int maxlen, int len);
+
 #endif /* _gnokii_internal_h */
