@@ -1,10 +1,10 @@
 /*
 
-  $Id: db.h,v 1.10 2003/10/05 19:42:41 ja Exp $
+  $Id: db.h,v 1.13 2006/08/10 19:10:14 pkot Exp $
 
   S M S D
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix tool for the mobile phones.
 
   This file is part of gnokii.
 
@@ -23,7 +23,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  & Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 1999-2005 Jan Derfinak
 
 */
 
@@ -34,10 +34,10 @@
 #include "smsd.h"
 #include "gnokii.h"
 
-extern void (*DB_Bye) (void);
-extern gint (*DB_ConnectInbox) (const DBConfig);
-extern gint (*DB_ConnectOutbox) (const DBConfig);
-extern gint (*DB_InsertSMS) (const gn_sms * const, const gchar * const);
-extern void (*DB_Look) (const gchar * const);
+GNOKII_API void (*DB_Bye) (void);
+GNOKII_API gint (*DB_ConnectInbox) (const DBConfig);
+GNOKII_API gint (*DB_ConnectOutbox) (const DBConfig);
+GNOKII_API gint (*DB_InsertSMS) (const gn_sms * const, const gchar * const);
+GNOKII_API void (*DB_Look) (const gchar * const);
 
 #endif
