@@ -1,10 +1,10 @@
 /*
 
-  $Id: xgnokii_netmon.c,v 1.11 2002/03/28 21:37:50 pkot Exp $
+  $Id: xgnokii_netmon.c,v 1.17 2006/08/15 10:57:20 dforsi Exp $
 
   X G N O K I I
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix GUI for the mobile phones.
 
   This file is part of gnokii.
 
@@ -23,7 +23,8 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  & Jn Derfik <ja@mail.upjs.sk>.
+  & 1999-2005 Jan Derfinak.
+  Copyright (C) 2002-2004 Pawel Kot
 
 */
 
@@ -42,14 +43,14 @@
 #include "xpm/Display.xpm"
 
 static GtkWidget *GUI_NetmonWindow;
-static DisplayData displayData = { NULL, 0 };
+static DisplayData displayData = { NULL, NULL, 0 };
 static GtkWidget *tableLabels[4][7];
 static GtkWidget *tableProgress[7];
 
 
 static inline void Help1(GtkWidget * w, gpointer data)
 {
-	gchar *indx = g_strdup_printf("/help/%s/windows/netmon/index.html", xgnokiiConfig.locale);
+	gchar *indx = g_strdup_printf("/help/%s/windows/netmon/index.html", xgnokiiConfig.help_locale);
 	Help(w, indx);
 	g_free(indx);
 }
@@ -230,7 +231,7 @@ void AddToBorder(GtkWidget * table, gchar * text, gint i, gint j, gint k, gint l
 	GtkWidget *label = gtk_label_new(text);
 	GtkWidget *frame = gtk_frame_new(NULL);
 
-	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
+//	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
 	gtk_container_add(GTK_CONTAINER(frame), label);
 	gtk_table_attach_defaults(GTK_TABLE(table), frame, i, j, k, l);
 }
@@ -244,7 +245,7 @@ inline void GUI_ShowNetmon()
 	e->event = Event_NetMonitor;
 	e->data = (gpointer) displayData.curDisp;;
 	GUI_InsertEvent(e);
-	gtk_widget_show(GUI_NetmonWindow);
+	gtk_window_present(GTK_WINDOW(GUI_NetmonWindow));
 }
 
 static GtkItemFactoryEntry menu_items[] = {
@@ -275,28 +276,6 @@ static void InitMainMenu(void)
 #define DISPLAY_X	12
 #define DISPLAY_Y	15
 
-static inline gint ExposeDisplay(GtkWidget * widget, GdkEventExpose * event)
-{
-	GdkRectangle rec;
-	GtkRequisition r;
-
-	gtk_widget_size_request(displayData.number, &r);
-	rec.x = DISPLAY_X;
-	rec.y = DISPLAY_Y;
-	rec.width = r.width;
-	rec.height = r.height;
-	gtk_widget_draw(displayData.number, &rec);
-
-	gtk_widget_size_request(displayData.label, &r);
-	rec.x += 3;
-	rec.y += 12;
-	rec.width = r.width;
-	rec.height = r.height;
-	gtk_widget_draw(displayData.label, &rec);
-
-	return FALSE;
-}
-
 
 void GUI_CreateNetmonWindow()
 {
@@ -331,7 +310,7 @@ void GUI_CreateNetmonWindow()
 
 	gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
 
-	gtk_accel_group_attach(accel_group, GTK_OBJECT(GUI_NetmonWindow));
+	gtk_window_add_accel_group(GTK_WINDOW(GUI_NetmonWindow), accel_group);
 
 	/* Finally, return the actual menu bar created by the item factory. */
 	menubar = gtk_item_factory_get_widget(item_factory, "<main>");
@@ -362,9 +341,6 @@ void GUI_CreateNetmonWindow()
 	pixmap = NewPixmap(Display_xpm, GUI_NetmonWindow->window,
 			   &GUI_NetmonWindow->style->bg[GTK_STATE_NORMAL]);
 
-	gtk_signal_connect_after(GTK_OBJECT(pixmap), "expose_event",
-				 (GtkSignalFunc) ExposeDisplay, NULL);
-
 	gtk_fixed_put(GTK_FIXED(display), pixmap, 0, 0);
 	gtk_widget_show(pixmap);
 
@@ -379,8 +355,9 @@ void GUI_CreateNetmonWindow()
 	vbox = gtk_vbox_new(TRUE, 0);
 
 	/* 1. line */
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NORMAL);
+	toolbar = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_TEXT);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "1", _("Active cell"),
 				NULL, NULL, (GtkSignalFunc) SetDisplay, (gpointer) 1);
@@ -397,7 +374,7 @@ void GUI_CreateNetmonWindow()
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "5", _("NCELL list III"),
 				NULL, NULL, (GtkSignalFunc) SetDisplay, (gpointer) 5);
 
-	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "6", _("Prefered/Denied networks"),
+	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "6", _("Preferred/Denied networks"),
 				NULL, NULL, (GtkSignalFunc) SetDisplay, (gpointer) 6);
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "7", _("System information bits"),
@@ -416,8 +393,9 @@ void GUI_CreateNetmonWindow()
 	gtk_widget_show(toolbar);
 
 	/* 2. line */
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NORMAL);
+	toolbar = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_TEXT);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "13", _("Uplink DTX"),
 				NULL, NULL, (GtkSignalFunc) SetDisplay, (gpointer) 13);
@@ -454,8 +432,9 @@ void GUI_CreateNetmonWindow()
 	gtk_widget_show(toolbar);
 
 	/* 3. line */
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NORMAL);
+	toolbar = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_TEXT);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "40", _("Reset handover counters"),
 				NULL, NULL, (GtkSignalFunc) SetDisplay, (gpointer) 40);
@@ -492,8 +471,9 @@ void GUI_CreateNetmonWindow()
 	gtk_widget_show(toolbar);
 
 	/* 4. line */
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NORMAL);
+	toolbar = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_TEXT);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), "62", _("Neighbourhood measurement"),
 				NULL, NULL, (GtkSignalFunc) SetDisplay, (gpointer) 62);
@@ -580,7 +560,7 @@ void GUI_CreateNetmonWindow()
 	button = gtk_spin_button_new(adj, 0, 0);
 	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(button), TRUE);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(button), TRUE);
-	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(button), GTK_SHADOW_OUT);
+//	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(button), GTK_SHADOW_OUT);
 	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
 			   GTK_SIGNAL_FUNC(ChangedSpinner), (gpointer) button);
 	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
